package com.gic.enterprise.service.outer.impl;

import com.alibaba.fastjson.JSON;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.commons.util.GICMQClientUtil;
import com.gic.enterprise.constant.AccountStandardEnum;
import com.gic.enterprise.constant.Constants;
import com.gic.enterprise.dto.DictDTO;
import com.gic.enterprise.dto.InternationalSmsRecordDTO;
import com.gic.enterprise.entity.TabBillingAccount;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import com.gic.enterprise.service.*;
import com.gic.mq.sdk.GicMQClient;
import com.gic.store.utils.valid.ValidUtil;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;

/**
 *
 * @Description:
 * @author zhiwj
 * @date 2019-12-09 9:26
 */
@Service("internationalSmsRecordApiService")
public class InternationalSmsRecordApiServiceImpl implements InternationalSmsRecordApiService {

    private Logger logger = LogManager.getLogger(InternationalSmsRecordApiServiceImpl.class);

    @Autowired
    private BillingAccountService billingAccountService;
    @Autowired
    private BillingInternationalSmsRecordService billingInternationalSmsRecordService;
    @Autowired
    private DictApiService dictApiService;
    @Autowired
    private AccountStandardService accountStandardService;

    @Override
    public ServiceResponse<Void> verifySendInternationalSms(Integer enterpriseId, Integer sendCount) {
        if (enterpriseId == null || sendCount == null) {
            return EnterpriseServiceResponse.failure(ErrorCode.PARAMETER_ERROR);
        }

        TabBillingAccount billingAccount = billingAccountService.getByEnterpriseId(enterpriseId);
        Double accountBalance = billingAccount.getAccountBalance();
        Integer unitFee = accountStandardService.queryUnitFee(enterpriseId, AccountStandardEnum.SMS.getCode());

        double needFee = (double) (sendCount * unitFee) / 1000;
        if (needFee - accountBalance > 5000d) {
            logger.info("enterpriseId:{}, 余额：{}, 发送条数：{}, 不能发送", enterpriseId, accountBalance, sendCount);
            return EnterpriseServiceResponse.failure(ErrorCode.OPERATION_FAILED.getCode(), "余额不足， 不能发送");
        } else {
            logger.info("enterpriseId:{}, 余额：{}, 发送条数：{}, 可以发送", enterpriseId, accountBalance, sendCount);
            return EnterpriseServiceResponse.success();
        }
    }

    @Override
    public ServiceResponse<Void> saveInternationalSmsRecord(InternationalSmsRecordDTO internationalSmsRecordDTO) {
        ServiceResponse paramResult = ValidUtil.allCheckValidate(internationalSmsRecordDTO, InternationalSmsRecordDTO.InternationalSmsRecordValid.class);
        if (!paramResult.isSuccess()) {
            return paramResult;
        }
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            String json = JSON.toJSONString(internationalSmsRecordDTO);
            logger.info("收到海外短信计费信息：{}", json);
            client.sendMessage("InternationalSmsMq", json);
//            this.run(json);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> run(String arg) {
        // 缓存中计费
        GicMQClient client = GICMQClientUtil.getClientInstance();
        try {
            logger.info("海外短信缓存扣费：{}", arg);
            InternationalSmsRecordDTO internationalSmsRecordDTO = JSON.parseObject(arg, InternationalSmsRecordDTO.class);
            // 判断是扣套餐包还是扣余额
            Integer fee = accountStandardService.queryUnitFee(internationalSmsRecordDTO.getEnterpriseId(), AccountStandardEnum.SMS.getCode());
            billingAccountService.deductAccountByCache(fee, internationalSmsRecordDTO.getEnterpriseId());
            internationalSmsRecordDTO.setFee((double)fee / 1000);

            Integer smsRecordId = billingInternationalSmsRecordService.save(internationalSmsRecordDTO);
            internationalSmsRecordDTO.setSmsRecordId(smsRecordId);

            // 队列
            client.sendMessage("InternationalSmsDbMq", JSON.toJSONString(internationalSmsRecordDTO));
            billingInternationalSmsRecordService.updateCacheStatus(internationalSmsRecordDTO.getSmsRecordId());
//            this.executeDatabase(arg);
        } catch (Exception e) {
            logger.warn("调用mq失败", e);
        }
        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> executeDatabase(String arg) {
        logger.info("海外短信数据库扣费：{}", arg);
        InternationalSmsRecordDTO internationalSmsRecordDTO = JSON.parseObject(arg, InternationalSmsRecordDTO.class);
        internationalSmsRecordDTO.setDeductionTime(new Date());
        ServiceResponse<DictDTO> serviceDict = dictApiService.getDict(Constants.SMS_APP_TYPE, internationalSmsRecordDTO.getAppCode());
        if (serviceDict.isSuccess()) {
            DictDTO dictDTO = serviceDict.getResult();
            if (dictDTO != null) {
                String key = dictDTO.getKey();
                internationalSmsRecordDTO.setAppName(key);
            }
        }
        billingAccountService.deductAccountByDatabase(internationalSmsRecordDTO.getFee(), internationalSmsRecordDTO.getEnterpriseId());
        billingInternationalSmsRecordService.updateDatabaseStatus(internationalSmsRecordDTO);
        return ServiceResponse.success();
    }
}
